@@ -286,13 +286,29 @@ tp_edge_scroll_init(struct tp_dispatch *tp, struct evdev_device *device)
 {
 	struct tp_touch *t;
 	int edge_width, edge_height;
+	double width, height;
+	bool want_horiz_scroll = true;
+
+	/* Touchpads smaller than 50mm are not tall enough to have a
+	   horizontal scroll area, it takes too much space away. But
+	   clickpads have enough space here anyway because of the
+	   software button area (and all these tiny clickpads were built
+	   when software buttons were a thing, e.g. Lenovo *20 series)
+	 */
+	if (!tp->buttons.is_clickpad) {
+	    evdev_device_get_size(device, &width, &height);
+	    want_horiz_scroll = (height >= 50);
+	}
 
 	/* 7mm edge size */
 	edge_width = device->abs.absinfo_x->resolution * 7;
 	edge_height = device->abs.absinfo_y->resolution * 7;
 
 	tp->scroll.right_edge = device->abs.absinfo_x->maximum - edge_width;
-	tp->scroll.bottom_edge = device->abs.absinfo_y->maximum - edge_height;
+	if (want_horiz_scroll)
+		tp->scroll.bottom_edge = device->abs.absinfo_y->maximum - edge_height;
+	else
+		tp->scroll.bottom_edge = INT_MAX;
 
 	tp_for_each_touch(tp, t) {
 		t->scroll.direction = -1;
@@ -318,6 +334,15 @@ tp_edge_scroll_handle_state(struct tp_dispatch *tp, uint64_t time)
 {
 	struct tp_touch *t;
 
+	if (tp->scroll.method != LIBINPUT_CONFIG_SCROLL_EDGE) {
+		tp_for_each_touch(tp, t) {
+			if (t->state == TOUCH_BEGIN)
+				t->scroll.edge_state =
+					EDGE_SCROLL_TOUCH_STATE_AREA;
+		}
+		return;
+	}
+
 	tp_for_each_touch(tp, t) {
 		if (!t->dirty)
 			continue;
@@ -350,9 +375,6 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 	const struct normalized_coords zero = { 0.0, 0.0 };
 	const struct discrete_coords zero_discrete = { 0.0, 0.0 };
 
-	if (tp->scroll.method != LIBINPUT_CONFIG_SCROLL_EDGE)
-		return 0;
-
 	tp_for_each_touch(tp, t) {
 		if (!t->dirty)
 			continue;
