@@ -93,7 +93,7 @@ tp_get_average_touches_delta(struct tp_dispatch *tp)
 static void
 tp_gesture_start(struct tp_dispatch *tp, uint64_t time)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 	const struct normalized_coords zero = { 0.0, 0.0 };
 
 	if (tp->gesture.started)
@@ -182,12 +182,15 @@ tp_gesture_get_active_touches(const struct tp_dispatch *tp,
 }
 
 static int
-tp_gesture_get_direction(struct tp_dispatch *tp, struct tp_touch *touch)
+tp_gesture_get_direction(struct tp_dispatch *tp, struct tp_touch *touch,
+			 unsigned int nfingers)
 {
 	struct normalized_coords normalized;
 	struct device_float_coords delta;
 	double move_threshold = TP_MM_TO_DPI_NORMALIZED(1);
 
+	move_threshold *= (nfingers - 1);
+
 	delta = device_delta(touch->point, touch->gesture.initial);
 
 	normalized = tp_normalize_delta(tp, delta);
@@ -330,24 +333,25 @@ tp_gesture_handle_state_unknown(struct tp_dispatch *tp, uint64_t time)
 
 	/* for two-finger gestures, if the fingers stay unmoving for a
 	 * while, assume (slow) scroll */
-	if (tp->gesture.finger_count == 2 &&
-	    time > (tp->gesture.initial_time + DEFAULT_GESTURE_2FG_SCROLL_TIMEOUT)) {
-		tp_gesture_set_scroll_buildup(tp);
-		return GESTURE_STATE_SCROLL;
-	}
+	if (tp->gesture.finger_count == 2) {
+		if (time > (tp->gesture.initial_time + DEFAULT_GESTURE_2FG_SCROLL_TIMEOUT)) {
+			tp_gesture_set_scroll_buildup(tp);
+			return GESTURE_STATE_SCROLL;
+		}
 
-	/* Else check if one finger is > 20mm below the others */
-	vert_distance = abs(first->point.y - second->point.y);
-	if (vert_distance > 20 * yres &&
-	    tp->gesture.finger_count > 2 &&
-	    tp->gesture.enabled) {
-		tp_gesture_init_pinch(tp);
-		return GESTURE_STATE_PINCH;
+		/* Else check if one finger is > 20mm below the others */
+		vert_distance = abs(first->point.y - second->point.y);
+		if (vert_distance > 20 * yres &&
+		    tp->gesture.finger_count > 2 &&
+		    tp->gesture.enabled) {
+			tp_gesture_init_pinch(tp);
+			return GESTURE_STATE_PINCH;
+		}
 	}
 
 	/* Else wait for both fingers to have moved */
-	dir1 = tp_gesture_get_direction(tp, first);
-	dir2 = tp_gesture_get_direction(tp, second);
+	dir1 = tp_gesture_get_direction(tp, first, tp->gesture.finger_count);
+	dir2 = tp_gesture_get_direction(tp, second, tp->gesture.finger_count);
 	if (dir1 == UNDEFINED_DIRECTION || dir2 == UNDEFINED_DIRECTION)
 		return GESTURE_STATE_UNKNOWN;
 
@@ -525,7 +529,7 @@ tp_gesture_stop_twofinger_scroll(struct tp_dispatch *tp, uint64_t time)
 static void
 tp_gesture_end(struct tp_dispatch *tp, uint64_t time, bool cancelled)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 	enum tp_gesture_state state = tp->gesture.state;
 
 	tp->gesture.state = GESTURE_STATE_NONE;
@@ -628,7 +632,7 @@ tp_init_gesture(struct tp_dispatch *tp)
 	tp->gesture.state = GESTURE_STATE_NONE;
 
 	libinput_timer_init(&tp->gesture.finger_count_switch_timer,
-			    tp->device->base.seat->libinput,
+			    tp_libinput_context(tp),
 			    tp_gesture_finger_count_switch_timeout, tp);
 	return 0;
 }
