@@ -382,6 +382,13 @@ extern struct litest_test_device litest_wacom_cintiq_24hd_device;
 extern struct litest_test_device litest_multitouch_fuzz_screen_device;
 extern struct litest_test_device litest_wacom_intuos3_pad_device;
 extern struct litest_test_device litest_wacom_intuos5_pad_device;
+extern struct litest_test_device litest_keyboard_all_codes_device;
+extern struct litest_test_device litest_magicmouse_device;
+extern struct litest_test_device litest_wacom_ekr_device;
+extern struct litest_test_device litest_wacom_cintiq_24hdt_pad_device;
+extern struct litest_test_device litest_wacom_cintiq_13hdt_finger_device;
+extern struct litest_test_device litest_wacom_cintiq_13hdt_pen_device;
+extern struct litest_test_device litest_wacom_cintiq_13hdt_pad_device;
 
 struct litest_test_device* devices[] = {
 	&litest_synaptics_clickpad_device,
@@ -430,6 +437,13 @@ struct litest_test_device* devices[] = {
 	&litest_multitouch_fuzz_screen_device,
 	&litest_wacom_intuos3_pad_device,
 	&litest_wacom_intuos5_pad_device,
+	&litest_keyboard_all_codes_device,
+	&litest_magicmouse_device,
+	&litest_wacom_ekr_device,
+	&litest_wacom_cintiq_24hdt_pad_device,
+	&litest_wacom_cintiq_13hdt_finger_device,
+	&litest_wacom_cintiq_13hdt_pen_device,
+	&litest_wacom_cintiq_13hdt_pad_device,
 	NULL,
 };
 
@@ -1156,6 +1170,58 @@ litest_restore_log_handler(struct libinput *libinput)
 	libinput_log_set_handler(libinput, litest_log_handler);
 }
 
+static inline int
+create_udev_lock_file(void)
+{
+	int lfd;
+
+	/* Running the multiple tests in parallel usually trips over udev
+	 * not being  up-to-date. We change the udev rules for every device
+	 * created, sometimes this means we end up getting the wrong udev
+	 * device, or having wrong properties applied.
+	 *
+	 * litests use the path interface and there is a window between
+	 * creating the device (which triggers udev reloads) and adding the
+	 * device to the libinput context where another udev reload may
+	 * upset things.
+	 *
+	 * To avoid this, create a lockfile on device add and device delete
+	 * to make sure that we have exclusive access to udev while
+	 * the udev rules are reloaded.
+	 */
+	do {
+		lfd = open(LITEST_UDEV_LOCKFILE, O_CREAT|O_EXCL, O_RDWR);
+
+		if (lfd == -1) {
+			struct stat st;
+			time_t now = time(NULL);
+
+			litest_assert_int_eq(errno, EEXIST);
+			msleep(10);
+
+			/* If the lock file is older than 10s, it's a
+			   leftover from some aborted test */
+			if (stat(LITEST_UDEV_LOCKFILE, &st) != -1) {
+				if (st.st_mtime < now - 10) {
+					fprintf(stderr,
+						"Removing stale lock file %s.\n",
+						LITEST_UDEV_LOCKFILE);
+					unlink(LITEST_UDEV_LOCKFILE);
+				}
+			}
+		}
+	} while (lfd < 0);
+
+	return lfd;
+}
+
+static inline void
+delete_udev_lock_file(int lfd)
+{
+	close(lfd);
+	unlink(LITEST_UDEV_LOCKFILE);
+}
+
 struct litest_device *
 litest_add_device_with_overrides(struct libinput *libinput,
 				 enum litest_device_type which,
@@ -1169,6 +1235,8 @@ litest_add_device_with_overrides(struct libinput *libinput,
 	int rc;
 	const char *path;
 
+	int lfd = create_udev_lock_file();
+
 	d = litest_create(which,
 			  name_override,
 			  id_override,
@@ -1194,6 +1262,9 @@ litest_add_device_with_overrides(struct libinput *libinput,
 		d->interface->min[ABS_Y] = libevdev_get_abs_minimum(d->evdev, ABS_Y);
 		d->interface->max[ABS_Y] = libevdev_get_abs_maximum(d->evdev, ABS_Y);
 	}
+
+	delete_udev_lock_file(lfd);
+
 	return d;
 }
 
@@ -1250,9 +1321,13 @@ litest_handle_events(struct litest_device *d)
 void
 litest_delete_device(struct litest_device *d)
 {
+	int lfd;
+
 	if (!d)
 		return;
 
+	lfd = create_udev_lock_file();
+
 	if (d->udev_rule_file) {
 		unlink(d->udev_rule_file);
 		free(d->udev_rule_file);
@@ -1270,6 +1345,8 @@ litest_delete_device(struct litest_device *d)
 	free(d->private);
 	memset(d,0, sizeof(*d));
 	free(d);
+
+	delete_udev_lock_file(lfd);
 }
 
 void
@@ -2279,6 +2356,10 @@ litest_create_uinput(const char *name,
 	abs = abs_info;
 	while (abs && abs->value != -1) {
 		if (abs->resolution != 0) {
+			if (libevdev_get_abs_resolution(dev, abs->value) ==
+			    abs->resolution)
+				break;
+
 			rc = libevdev_kernel_set_abs_info(dev,
 							  abs->value,
 							  abs);
@@ -2325,7 +2406,7 @@ litest_create_uinput_device_from_description(const char *name,
 	syspath = libevdev_uinput_get_syspath(uinput);
 
 	/* blocking, we don't want to continue until udev is ready */
-	do {
+	while (1) {
 		udev_device = udev_monitor_receive_device(udev_monitor);
 		litest_assert_notnull(udev_device);
 		udev_action = udev_device_get_action(udev_device);
@@ -2335,7 +2416,11 @@ litest_create_uinput_device_from_description(const char *name,
 		}
 
 		udev_syspath = udev_device_get_syspath(udev_device);
-	} while (!udev_syspath || strcmp(udev_syspath, syspath) != 0);
+		if (udev_syspath && streq(udev_syspath, syspath))
+			break;
+
+		udev_device_unref(udev_device);
+	}
 
 	litest_assert(udev_device_get_property_value(udev_device, "ID_INPUT"));
 
@@ -3022,7 +3107,7 @@ main(int argc, char **argv)
 
 	list_init(&all_tests);
 
-	setenv("CK_DEFAULT_TIMEOUT", "10", 0);
+	setenv("CK_DEFAULT_TIMEOUT", "30", 0);
 	setenv("LIBINPUT_RUNNING_TEST_SUITE", "1", 1);
 
 	mode = litest_parse_argv(argc, argv);
