@@ -230,6 +230,22 @@ print_device_notify(struct libinput_event *ev)
 			printf(" dwt-off)");
 	}
 
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_TABLET_PAD)) {
+		int nbuttons, nstrips, nrings, ngroups;
+
+		nbuttons = libinput_device_tablet_pad_get_num_buttons(dev);
+		nstrips = libinput_device_tablet_pad_get_num_strips(dev);
+		nrings = libinput_device_tablet_pad_get_num_rings(dev);
+		ngroups = libinput_device_tablet_pad_get_num_mode_groups(dev);
+
+		printf(" buttons:%d strips:%d rings:%d mode groups:%d",
+		       nbuttons,
+		       nstrips,
+		       nrings,
+		       ngroups);
+	}
+
 	printf("\n");
 
 }
@@ -590,14 +606,25 @@ static void
 print_tablet_pad_button_event(struct libinput_event *ev)
 {
 	struct libinput_event_tablet_pad *p = libinput_event_get_tablet_pad_event(ev);
+	struct libinput_tablet_pad_mode_group *group;
 	enum libinput_button_state state;
+	unsigned int button, mode;
 
 	print_event_time(libinput_event_tablet_pad_get_time(p));
 
+	button = libinput_event_tablet_pad_get_button_number(p),
 	state = libinput_event_tablet_pad_get_button_state(p);
-	printf("%3d %s\n",
-	       libinput_event_tablet_pad_get_button_number(p),
-	       state == LIBINPUT_BUTTON_STATE_PRESSED ? "pressed" : "released");
+	mode = libinput_event_tablet_pad_get_mode(p);
+	printf("%3d %s (mode %d)",
+	       button,
+	       state == LIBINPUT_BUTTON_STATE_PRESSED ? "pressed" : "released",
+	       mode);
+
+	group = libinput_event_tablet_pad_get_mode_group(p);
+	if (libinput_tablet_pad_mode_group_button_is_toggle(group, button))
+		printf(" <mode toggle>");
+
+	printf("\n");
 }
 
 static void
@@ -605,6 +632,7 @@ print_tablet_pad_ring_event(struct libinput_event *ev)
 {
 	struct libinput_event_tablet_pad *p = libinput_event_get_tablet_pad_event(ev);
 	const char *source = "<invalid>";
+	unsigned int mode;
 
 	print_event_time(libinput_event_tablet_pad_get_time(p));
 
@@ -617,10 +645,12 @@ print_tablet_pad_ring_event(struct libinput_event *ev)
 		break;
 	}
 
-	printf("ring %d position %.2f (source %s)\n",
+	mode = libinput_event_tablet_pad_get_mode(p);
+	printf("ring %d position %.2f (source %s) (mode %d)\n",
 	       libinput_event_tablet_pad_get_ring_number(p),
 	       libinput_event_tablet_pad_get_ring_position(p),
-	       source);
+	       source,
+	       mode);
 }
 
 static void
@@ -628,6 +658,7 @@ print_tablet_pad_strip_event(struct libinput_event *ev)
 {
 	struct libinput_event_tablet_pad *p = libinput_event_get_tablet_pad_event(ev);
 	const char *source = "<invalid>";
+	unsigned int mode;
 
 	print_event_time(libinput_event_tablet_pad_get_time(p));
 
@@ -640,10 +671,12 @@ print_tablet_pad_strip_event(struct libinput_event *ev)
 		break;
 	}
 
-	printf("strip %d position %.2f (source %s)\n",
+	mode = libinput_event_tablet_pad_get_mode(p);
+	printf("strip %d position %.2f (source %s) (mode %d)\n",
 	       libinput_event_tablet_pad_get_strip_number(p),
 	       libinput_event_tablet_pad_get_strip_position(p),
-	       source);
+	       source,
+	       mode);
 }
 
 static int
